Option Explicit ' Make sure that variables must be declared before use

Begin Dialog CALENDAR 147, 47, 149, 75, "Calendar"
	Text 12, 8, 124, 12, "For which year do you wish to create a calendar?"
	TextBox 12, 20, 48, 10, .Year
	OptionGroup .holidays
		OptionButton 12, 32, 84, 12, "Include German holidays"
		OptionButton 12, 40, 84, 12, "Without German holidays"
	OKButton 20, 60, 48, 12
	CancelButton 76, 60, 48, 12
End Dialog

Dim pm As Object                   ' PlanMaker object
Dim vDays(7) As String             ' Weekday names
Dim vMonthDay(1 To 12) As Integer  ' Number of days per month
Dim GetYear As CALENDAR            ' Dialog
Dim UseHolidays As Integer         ' With or without holidays?
Dim CalYear As Integer             ' Desired year
Dim HolidayEaster As Integer       ' Easter Sunday: nth day in the year
Dim HolidayChristmas As Integer    ' 4th sunday in Advent: nth day in the year
Dim HolidaySummer As Integer       ' Begin of summertime
Dim HolidayWinter As Integer       ' End of summertime
Dim HolidayMothersDay As Integer   ' Mother's Day

Sub Main
   Dim vMonths(1 To 12) As String  ' Month names
   Dim Cell As String              ' Current cell
   Dim Start As String             ' Starting cell of a month
   Dim CalWeekDay As Integer       ' Current weekday
   Dim CalMonth As Integer         ' Loop counter for months
   Dim LeapYear As Integer         ' Is this year a leap year?
   Dim CalNumberDay As Integer     ' Current day
   Dim CurRange As Object
 
	' Initialize Arrays
 
	vDays(0) = "So"
	vDays(1) = "Mo"
	vDays(2) = "Di"
	vDays(3) = "Mi"
	vDays(4) = "Do"
	vDays(5) = "Fr"
	vDays(6) = "Sa"
	
	vMonths(1) = "January"
	vMonths(2) = "February"
	vMonths(3) = "March"
	vMonths(4) = "April"
	vMonths(5) = "May"
	vMonths(6) = "June"
	vMonths(7) = "July"
	vMonths(8) = "August"
	vMonths(9) = "September"
	vMonths(10) = "October"
	vMonths(11) = "November"
	vMonths(12) = "December"

	vMonthDay(1) = 31
	vMonthDay(2) = 28
	vMonthDay(3) = 31
	vMonthDay(4) = 30
	vMonthDay(5) = 31
	vMonthDay(6) = 30
	vMonthDay(7) = 31
	vMonthDay(8) = 31
	vMonthDay(9) = 30
	vMonthDay(10) = 31
	vMonthDay(11) = 30
	vMonthDay(12) = 31

	' Set default values for the dialog
	GetYear.Year = year(now())		'2013
	GetYear.Holidays = 0

	' Show dialog
	If Dialog(GetYear) = 0 Then Exit Sub

	CalYear = GetYear.Year
	UseHolidays = GetYear.Holidays
 
	' Starting cell
	Cell = "A2"

	' First weekday of the year
	CalWeekDay = Weekday("01.01." + Str(CalYear)) - 1

	' Current day in the year
	CalNumberDay = 1

	' Is this year a leap year?
	If (CalYear Mod 4 = 0) And (CalYear Mod 100 <> 0) Then
		LeapYear = 1
	Else 
		LeapYear = 0
	End If
 
	If CalYear Mod 400 = 0 Then LeapYear = 1

	vMonthDay(2) = 28 + LeapYear

	' Find out the day in the year for Easter Sunday
	HolidayEaster = GetHolidayEaster(CalYear, LeapYear)

	' Find out the day for the 4th Sunday in Advent in this year
	HolidayChristmas = 359 + LeapYear - Weekday("24.12." + Str(CalYear))

	' Find out the day in this year for Mother's Day
	HolidayMothersDay = 135 + LeapYear - Weekday("30.04." + Str(CalYear))
	If HolidayMothersDay = HolidayEaster + 49 Then ' Mother's Day <> Pentecoste Sunday
		HolidayMothersDay = HolidayMothersDay - 7
	End If

	' Launch PlanMaker
	Set pm = CreateObject("PlanMaker.Application")

	' Create a new document
	pm.Visible = True
	pm.Activate
	pm.Workbooks.Add

	' Turn off automatic functions to increase execution speed
	pm.ActiveWorkbook.Calculation = pmCalculationManual
	pm.ActiveWindow.DisplayGridlines = False
	pm.ActiveWorkbook.ScreenUpdate = False
	pm.ActiveWorkbook.ManualApply = True

	' Heading
	Set CurRange = pm.Range("A1")
	CurRange.Value = "Calendar " + Str(CalYear)
	CurRange.Font.Bold = True
	CurRange.Font.Size = 28
	CurRange.Font.Name = "Arial"
	CurRange.ApplyFormatting

	For CalMonth = 1 To 12
		Start = Cell

		' Fill in the month
		FillMonth Cell, CalWeekDay, CalMonth, CalNumberDay

		' Month name
		Set CurRange = pm.Range(Cell)
		CurRange.Value = vMonths(CalMonth)        
		pm.Columns(CurRange.Column).ColumnWidth = 20

		Cell = NextCol(Cell)

		Set CurRange = pm.Range(Cell)
		pm.Columns(CurRange.Column).ColumnWidth = 18

		Cell = NextCol(Cell)

		Set CurRange = pm.Range(Cell)
		pm.Columns(CurRange.Column).ColumnWidth = 80

		' Format the month name
		Set CurRange = pm.Range(Start + ":" + Cell)

		CurRange.Font.Bold = True
		CurRange.Font.Size = 9
		CurRange.Font.Name = "Arial"

		CurRange.Shading.ForegroundPatternColor = pmColorCyan
		CurRange.Shading.Intensity = 30

		CurRange.Borders(pmBorderLeft).Type = 1
		CurRange.Borders(pmBorderLeft).Thick1 = 1

		CurRange.Borders(pmBorderTop).Type = 1
		CurRange.Borders(pmBorderTop).Thick1 = 1

		CurRange.Borders(pmBorderRight).Type = 1
		CurRange.Borders(pmBorderRight).Thick1 = 1

		CurRange.Borders(pmBorderBottom).Type = 1
		CurRange.Borders(pmBorderBottom).Thick1 = 1

		CurRange.LeftPadding = 4
		CurRange.ApplyFormatting

		Cell = NextCol(Cell)
	Next CalMonth

	If UseHolidays = 0 Then
		Set CurRange = pm.Range("A35")
		CurRange.Value = "* Holiday in some German states"
		CurRange.Font.Size = 8
		CurRange.ApplyFormatting
	End If

	Set CurRange = pm.Range("A37")
	CurRange.Value = "No responsibility is taken for the correctness of this information"
	CurRange.Font.Size = 8
	CurRange.ApplyFormatting

	pm.Range("A1").Select

	' Turn screen updates back on
	pm.ActiveWorkbook.Calculation = pmCalculationAutomatic
	pm.ActiveWorkbook.ScreenUpdate = True
End Sub

' Find out Easter Sunday

Function GetHolidayEaster(CYear As Integer, LeapYear As Integer) As Integer
	Dim gz As Integer
	Dim jhd As Integer
	Dim ksj As Integer
	Dim korr As Integer
	Dim so As Integer
	Dim epakte As Integer
	Dim n As Integer

	gz = (CYear Mod 19) + 1
	jhd = Int(CYear / 100) + 1
	ksj = Int((3 * jhd) / 4) - 12
	korr = Int((8 * jhd + 5) / 25) - 5
	so = int((5 * CYear) / 4) - ksj - 10
	epakte = (11 * gz + 20 + korr - ksj) Mod 30
	If (epakte = 25 And gz > 11) Or epakte = 24 Then epakte = epakte + 1
	n = 44 - epakte
	If n < 21 Then n = n + 30 
	n = n + 7 - (so + n) Mod 7

	GetHolidayEaster = n + LeapYear + 59
End Function

' Insert the holidays into the calendar

Function IsHoliday(Cell1 As String, Cell2 As String, CMonthDay As Integer, CMonth As Integer, NumberDay As Integer) As Integer
	IsHoliday = 0

	Select Case NumberDay
		Case HolidayEaster - 48
			pm.Range(Cell2).Value = "Carnival Monday"
			IsHoliday = 0
		Case HolidayEaster - 46
			pm.Range(Cell2).Value = "Ash Wednesday"
			IsHoliday = 0
		Case HolidayEaster - 3
			pm.Range(Cell2).Value = "Holy Thursday"
			IsHoliday = 0
		Case HolidayEaster - 2
			pm.Range(Cell2).Value = "Good Friday"
			IsHoliday = 1
		Case HolidayEaster
			pm.Range(Cell2).Value = "Easter Sunday"
			IsHoliday = 1
		Case HolidayEaster + 1
			pm.Range(Cell2).Value = "Easter Monday"
			IsHoliday = 1
		Case HolidayEaster + 39
			pm.Range(Cell2).Value = "Ascension Day"
			IsHoliday = 1
		Case HolidayEaster + 49
			pm.Range(Cell2).Value = "Whit Sunday"
			IsHoliday = 1
		Case HolidayEaster + 50
			pm.Range(Cell2).Value = "Whit Monday"
			IsHoliday = 1
		Case HolidayEaster + 60
			pm.Range(Cell2).Value = "Corpus Christi*"
			IsHoliday = 2
		Case HolidayChristmas - 35
			pm.Range(Cell2).Value = "Remembrance Day"
			IsHoliday = 1
		Case HolidayChristmas - 32
			pm.Range(Cell2).Value = "Penance Day*"
			IsHoliday = 2
		Case HolidayChristmas - 28
			pm.Range(Cell2).Value = "Commemorating the dead"
			IsHoliday = 1
		Case HolidayChristmas - 21
			pm.Range(Cell2).Value = "1st Sunday in Advent"
			IsHoliday = 0
		Case HolidayChristmas - 14
			pm.Range(Cell2).Value = "2nd Sunday in Advent"
			IsHoliday = 0
		Case HolidayChristmas - 7
			pm.Range(Cell2).Value = "3rd Sunday in Advent"
			IsHoliday = 0
		Case HolidayChristmas
			pm.Range(Cell2).Value = "4th Sunday in Advent"
			IsHoliday = 0
		Case HolidayMothersDay
			pm.Range(Cell2).Value = "Mother's Day"
			IsHoliday = 1
		Case Else
	End Select

	If CMonth = 3 And CMonthDay > 24 Then
		If WeekDay(Str(CMonthDay) + ".03." + Str(CalYear)) = 1 Then
			pm.Range(Cell2).Value = "Begin of summertime"
			IsHoliday = 0
		End If
	End If

	If CMonth = 10 And CMonthDay > 24 Then
		If WeekDay(Str(CMonthDay) + ".10." + Str(CalYear)) = 1 Then
			pm.Range(Cell2).Value = "End of summertime"
			IsHoliday = 0
		End If
	End If

	If CMonth = 10 And CMonthDay < 8 Then
		If WeekDay(Str(CMonthDay) + ".10." + Str(CalYear)) = 1 Then
			pm.Range(Cell2).Value = "Thanksgiving Day"
			IsHoliday = 1
		End If
	End If

	If CMonth = 1 And CMonthDay = 1 Then
		pm.Range(Cell2).Value = "New Year"
		IsHoliday = 1
	End If

	If CMonth = 1 And CMonthDay = 6 Then
		pm.Range(Cell2).Value = "Epiphany*"
		IsHoliday = 2
	End If

	If CMonth = 5 And CMonthDay = 1 Then
		pm.Range(Cell2).Value = "Intern'l Workers' Day"
		IsHoliday = 1
	End If

	If CMonth = 8 And CMonthDay = 15 Then
		pm.Range(Cell2).Value = "Assumption Day*"
		IsHoliday = 2
	End If

	If CMonth = 10 And CMonthDay = 3 Then
		pm.Range(Cell2).Value = "German Unification Day"
		IsHoliday = 1
	End If

	If CMonth = 10 And CMonthDay = 31 Then
		pm.Range(Cell2).Value = "Reformation Day*"
		IsHoliday = 2
	End If

	If CMonth = 11 And CMonthDay = 1 Then
		pm.Range(Cell2).Value = "All Saints' Day*"
		IsHoliday = 2
	End If

	If CMonth = 12 And CMonthDay = 6 Then
		pm.Range(Cell2).Value = "St. Nicholas Day"
		IsHoliday = 0
	End If

	If CMonth = 12 And CMonthDay = 24 Then
		pm.Range(Cell2).Value = "Christmas Eve"
		IsHoliday = 0
	End If

	If CMonth = 12 And CMonthDay = 25 Then
		pm.Range(Cell2).Value = "Christmas Day"
		IsHoliday = 1
	End If

	If CMonth = 12 And CMonthDay = 26 Then
		pm.Range(Cell2).Value = "Boxing Day"
		IsHoliday = 1
	End If

	If CMonth = 12 And CMonthDay = 31 Then
		pm.Range(Cell2).Value = "New Year's Eve"
		IsHoliday = 0
	End If

	If IsHoliday = 1 Then
		Set CurRange = pm.Range(Cell1 + ":" + Cell2)
		CurRange.Shading.ForegroundPatternColor = pmColorYellow
		CurRange.Shading.Intensity = 30
		CurRange.ApplyFormatting
	ElseIf IsHoliday = 2 Then
		Set CurRange = pm.Range(Cell1 + ":" + NextCol(Cell1))
		CurRange.Shading.ForegroundPatternColor = pmColorYellow
		CurRange.Shading.Intensity = 30
		CurRange.ApplyFormatting
	End If
End Function

' Create cells for a whole month

Sub FillMonth(StartCell As String, CDay As Integer, CMonth As Integer, NumberDay As Integer)
	Dim ColCell As String
	Dim RowCell As String
	Dim dstart As String
	Dim Cell2 As String
	Dim MonthDay As Integer
	Dim DrawWeekend As Integer

	RowCell = StartCell

	For MonthDay = 1 To vMonthDay(CMonth)
		ColCell = NextRow(RowCell)
		RowCell = ColCell

		DStart = ColCell

		' Enter weekday
		pm.Range(ColCell).value = vDays(cDay)
		ColCell = NextCol(ColCell)
		Cell2 = ColCell

		' Enter day of the monthz
		pm.Range(ColCell).value = Str(MonthDay)
		ColCell = NextCol(ColCell)

		DrawWeekEnd = 1

		If UseHolidays = 0 Then ' Holidays requested?
			If IsHoliday(DStart, ColCell, MonthDay, CMonth, NumberDay) > 0 Then DrawWeekEnd = 0
		End If

		If DrawWeekend Then
			If cDay = 6 Then
				Set CurRange = pm.Range(DStart)
				CurRange.Shading.ForegroundPatternColor = pmColorGreen
				CurRange.Shading.Intensity = 30
				CurRange.ApplyFormatting
			ElseIf cDay = 0 Then
				Set CurRange = pm.Range(DStart + ":" + ColCell)
				CurRange.Shading.ForegroundPatternColor = pmColorGreen
				CurRange.Shading.Intensity = 30
				CurRange.ApplyFormatting
			End If
		End If

		cDay = cDay + 1
		If cDay > 6 Then cDay = 0
		NumberDay = NumberDay + 1
	Next

	RowCell = NextRow(StartCell)

	' Borders around the weekday

	Set CurRange = pm.Range(RowCell + ":" + Cell2)
	CurRange.Font.Size = 8
	CurRange.Font.Name = "Arial"
	CurRange.Borders(pmBorderLeft).Type = 1
	CurRange.Borders(pmBorderLeft).Thick1 = 1
	CurRange.Borders(pmBorderTop).Type = 1
	CurRange.Borders(pmBorderTop).Thick1 = 1
	CurRange.Borders(pmBorderRight).Type = 1
	CurRange.Borders(pmBorderRight).Thick1 = 1
	CurRange.Borders(pmBorderBottom).Type = 1
	CurRange.Borders(pmBorderBottom).Thick1 = 1
	CurRange.Borders(pmBorderHorizontal).Type = 1
	CurRange.Borders(pmBorderHorizontal).Thick1 = 1
	CurRange.LeftPadding = 4
	CurRange.ApplyFormatting

	' Borders around the description of the weekday

	RowCell = NextCol(RowCell)
	RowCell = NextCol(RowCell)
	Set CurRange = pm.Range(RowCell + ":" + ColCell)
	CurRange.Font.Size = 8
	CurRange.Font.Name = "Arial"
	CurRange.Borders(pmBorderLeft).Type = 1
	CurRange.Borders(pmBorderLeft).Thick1 = 1
	CurRange.Borders(pmBorderTop).Type = 1
	CurRange.Borders(pmBorderTop).Thick1 = 1
	CurRange.Borders(pmBorderRight).Type = 1
	CurRange.Borders(pmBorderRight).Thick1 = 1
	CurRange.Borders(pmBorderBottom).Type = 1
	CurRange.Borders(pmBorderBottom).Thick1 = 1
	CurRange.Borders(pmBorderHorizontal).Type = 1
	CurRange.Borders(pmBorderHorizontal).Thick1 = 1
	CurRange.LeftPadding = 4
	CurRange.ApplyFormatting
End Sub

' Accepts a cell address and returns the address of the cell in the next row

Function NextRow(cell As String) As String
	Dim letters As Integer

	If InStr(1, "ABCDEFGHIJKLMNOPQRSTUVWXYZ", Mid(cell, 2, 1)) Then
		letters = 2
	Else
		letters = 1
	End If

	If letters = 2 Then
		NextRow = Left(cell, 2) + Str(Val(Mid(cell, 3, 8)) + 1)
	Else
		NextRow = Left(cell, 1) + Str(Val(Mid(cell, 2, 8)) + 1)
	End If
End Function

' Accepts a cell address and returns the address of the cell in the next column

Function NextCol(cell As String) As String
	Dim letters As Integer

	If InStr(1, "ABCDEFGHIJKLMNOPQRSTUVWXYZ", Mid(cell, 2, 1)) Then 
		letters = 2 
	Else
		letters = 1
	End If

	If Mid(cell, letters, 1) = "Z" Then
		If letters = 2 Then ' AZ1 -> BA1
			NextCol = Chr(Asc(Left(cell, 1)) + 1) + "A" + Mid(cell, 3, 8)
		Else ' Z1 -> AA1
			NextCol = "AA" + Mid(cell, 2, 8)
		End If
	Else
		If letters = 2 Then ' AA1 -> AB1
			NextCol = Left(cell, 1) + Chr(Asc(Mid(cell, 2, 1)) + 1) + Mid(cell, 3, 8)
		Else ' A1 -> B1
			NextCol = Chr(Asc(Left(cell, 1)) + 1) + Mid(cell, 2, 8)
		End If
	End If
End Function
